@@ -40,6 +40,10 @@
  */
 #define _XOPEN_SOURCE 700
 
+#if defined(HAVE_ISSETUGID) && defined(__FreeBSD__)
+#define __BSD_VISIBLE 1
+#endif
+
 #include "compat.h"
 #include "debug.h"
 
@@ -171,7 +175,7 @@ p11_mutex_init (p11_mutex_t *mutex)
 	int ret;
 
 	pthread_mutexattr_init (&attr);
-	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_DEFAULT);
+	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
 	ret = pthread_mutex_init (mutex, &attr);
 	assert (ret == 0);
 	pthread_mutexattr_destroy (&attr);
@@ -274,12 +278,6 @@ p11_dl_error (void)
 	return msg_buf;
 }
 
-void
-p11_dl_close (void *dl)
-{
-	FreeLibrary (dl);
-}
-
 int
 p11_thread_create (p11_thread_t *thread,
                    p11_thread_routine routine,
@@ -489,6 +487,23 @@ strndup (const char *data,
 
 #endif /* HAVE_STRNDUP */
 
+#ifndef HAVE_REALLOCARRAY
+
+void *
+reallocarray (void *ptr,
+	      size_t nmemb,
+	      size_t size)
+{
+	assert (nmemb > 0 && size > 0);
+	if (SIZE_MAX / nmemb < size) {
+		errno = ENOMEM;
+		return NULL;
+	}
+	return realloc (ptr, nmemb * size);
+}
+
+#endif /* HAVE_MEMDUP */
+
 #ifndef HAVE_STRCONCAT
 
 #include <stdarg.h>
@@ -668,7 +683,7 @@ _gettemp (char *path,
 
 	/* Fill space with random characters */
 	while (trv >= path && *trv == 'X') {
-		rnd = rand () % sizeof (padchar) - 1;
+		rnd = rand () % (sizeof (padchar) - 1);
 		*trv-- = padchar[rnd];
 	}
 	start = trv + 1;
@@ -857,6 +872,17 @@ strerror_r (int errnum,
 
 #endif /* HAVE_STRERROR_R */
 
+void
+p11_dl_close (void *dl)
+{
+#ifdef OS_WIN32
+	FreeLibrary (dl);
+#else
+	(void) dlclose (dl);
+#endif
+}
+
+
 #ifdef OS_UNIX
 
 #include <unistd.h>
